@@ -33,6 +33,16 @@ from tw.forms import TableForm
 import sqlalchemy
 from dbsprockets.util import freshdefaults
 
+from warnings import warn
+
+import sqlalchemy
+saVersion = int(sqlalchemy.__version__.split('.')[1])
+
+if saVersion>4:
+    from sqlalchemy.orm.attributes import ClassManager
+else:
+    warn('Versions of dbsprockets>=0.5 will require sqlalchemy>=0.5')
+
 class DBHelper:
     def __init__(self):
         self._identifiers = {}
@@ -41,25 +51,30 @@ class DBHelper:
 
     def getMetadata(self, model):
         raise NotImplementedError
-    
+
     def getIdentifier(self, model):
         raise NotImplementedError
 
     def validateModel(self, model):
         raise NotImplementedError
-    
+
 class _SAORMDBHelper(DBHelper):
-        
+
     def _firstColumn(self, model):
-        return model.c[model.c.keys()[0]]
-    
+#        import pdb; pdb.set_trace();
+        if saVersion<5 and hasattr(model, 'c'):
+            warn('Versions of dbsprockets>=0.5 will require sqlalchemy>=0.5')
+            return model.c[model.c.keys()[0]]
+        mapper = model._sa_class_manager.mappers.values()[0]
+        return mapper.c[mapper.c.keys()[0]]
+
     def getMetadata(self, model):
         firstColumn = self._firstColumn(model)
         if self._metadata is None:
             self._metadata = firstColumn.metadata
             self._identifiers[model] = firstColumn.table.name
         return self._metadata
-    
+
     def getIdentifier(self, model):
         if model not in self._identifiers:
             self._identifiers[model] = self._firstColumn(model).table.name
@@ -69,13 +84,17 @@ class _SAORMDBHelper(DBHelper):
         if self._provider is None:
             self._provider = SAProvider(self.getMetadata(model))
         return self._provider
-    
+
     def validateModel(self, model):
-        if not hasattr(model, 'c'):
-            raise TypeError('arg1(model) has not been mapped to an sql table appropriately')
-        
+        if hasattr(model, 'c') and isinstance(model.c, sqlalchemy.util.OrderedProperties):
+            warn('Versions of dbsprockets>=0.5 will require sqlalchemy>=0.5')
+            return
+
+        if not hasattr(model, '_sa_class_manager') or not isinstance(model._sa_class_manager, ClassManager):
+            raise TypeError('arg1(%s) has not been mapped to an sql table appropriately'%model)
+
         #failsafe, return a form with nothing in it
-        if len(model.c.keys()) == 0:
+        if len(model._sa_class_manager.mappers.values()[0].c.keys()) == 0:
             raise Exception('Form has no fields')
 
 SAORMDBHelper = _SAORMDBHelper()
@@ -87,37 +106,43 @@ SAORMDBHelper = _SAORMDBHelper()
 class DatabaseMixin:
     def __init__(self, dbHelper=None):
         self._dbHelper = dbHelper
-        
+
     def _getHelper(self, model):
         if self._dbHelper is None:
-            #use a SA Helper
+
+            #sa 0.4 way of doing things
             if hasattr(model, 'c') and isinstance(model.c, sqlalchemy.util.OrderedProperties):
+                warn('Versions of dbsprockets>=0.5 will require sqlalchemy>=0.5')
+                self._dbHelper = SAORMDBHelper
+
+            #use a SA Helper
+            elif hasattr(model, '_sa_class_manager') and isinstance(model._sa_class_manager, ClassManager):
                 self._dbHelper = SAORMDBHelper
             #other helper definitions are going in here
             else:
                 raise Exception('arg1(model) has not been mapped to an sql table appropriately')
         return self._dbHelper
-    
+
     def _validateModel(self, model):
         helper = self._getHelper(model)
         helper.validateModel(model)
 
 class SprocketGenerator(DatabaseMixin):
-    
-    def makeForm(self, 
-                 model, 
-                 action, 
+
+    def makeForm(self,
+                 model,
+                 action,
                  identifier='',
                  controller='/',
                  hiddenFields=None,
                  disabledFields=None,
-                 requiredFields=None, 
+                 requiredFields=None,
                  omittedFields=None,
                  additionalFields=None,
                  limitFields=None,
                  fieldAttrs=None,
-                 widgets=None, 
-                 validators=None, 
+                 widgets=None,
+                 validators=None,
                  formWidgetType=None,
                  formValidator=None,
                  checkIfUnique=False,
@@ -130,32 +155,32 @@ class SprocketGenerator(DatabaseMixin):
                  controller='/',
                  hiddenFields=[],
                  disabledFields=[],
-                 requiredFields=[], 
+                 requiredFields=[],
                  omittedFields=[],
                  additionalFields=[],
                  limitFields=None,
                  fieldAttrs={},
                  widgets={},
-                 validators={}, 
+                 validators={},
                  formWidgetType=None,
                  formValidator=None
                  checkIfUnique=False)-->tw.forms.TableForm
-                 
+
         Generates an entry form for use in web applications.
-        
+
         Arguments:
         name              type                           default    Description
         model             class                                     sqlalchemy mapped class definition
         action            String                                    string passed to the forms 'action'
         identifier        String or None                 None       optional identifier for the form
-        hiddenFields      list if names                  []         fields that are hidden but not altogether 
-                                                                    removed from the table, useful for passing 
+        hiddenFields      list if names                  []         fields that are hidden but not altogether
+                                                                    removed from the table, useful for passing
                                                                     data through a form manually.
         disabledFields    list of names                  []         fields that are shown but not editable
         requiredFields    list of names                  []         fields that cannot be left empty by the user (validation type stuff)
         omittedFields     list of names                  []         fields that are not present, whatsoever
         additionalFields  list of Toscawidgets.fields    []         fields that you would like to add to the form that are not in the model
-        limitFields       list of names                  None       limit the fields to the one in the list.  A side effect is that the 
+        limitFields       list of names                  None       limit the fields to the one in the list.  A side effect is that the
                                                                     order of the fields is set by this argument.
                                                                     (a good way to order your fields if you include all of them)
         fieldAttrs        dict of field Attrs            {}         a set of extra parameters which can be passed into the widgets
@@ -183,8 +208,8 @@ class SprocketGenerator(DatabaseMixin):
             widgets={}
         if validators is None:
             validators={}
-        
-        
+
+
         if not isinstance(action, types.StringTypes):
             raise TypeError('arg2(action) must be of type String, not %s'%type(action))
         if not isinstance(controller, types.StringTypes):
@@ -209,18 +234,18 @@ class SprocketGenerator(DatabaseMixin):
             raise TypeError('arg12(formValidator) must be of type Dict, not %s'%type(formValidator))
         if not isinstance(checkIfUnique, bool):
             raise TypeError('arg13(checkIfUnique) must be of type bool, not %s'%type(checkIfUnique))
-        
+
         self._validateModel(model)
-        
-        view = self.makeView(EditableRecordViewConfig, 
-                        model, 
+
+        view = self.makeView(EditableRecordViewConfig,
+                        model,
                         action,
                         identifier,
                         '',
-                        hiddenFields, 
+                        hiddenFields,
                         disabledFields,
-                        requiredFields, 
-                        omittedFields, 
+                        requiredFields,
+                        omittedFields,
                         additionalFields,
                         limitFields,
                         fieldAttrs,
@@ -230,32 +255,32 @@ class SprocketGenerator(DatabaseMixin):
                         formValidator,
                         checkIfUnique)
         return view.widget
-    
-    def makeRecordView(self, 
-                 model, 
+
+    def makeRecordView(self,
+                 model,
                  controller='',
                  omittedFields=None,
                  additionalFields=None,
                  limitFields=None,
                  fieldAttrs=None,
-                 widgets=None, 
-                 validators=None, 
+                 widgets=None,
+                 validators=None,
                  formWidgetType=None,
                  ):
         """
-        makeRecordView(model, 
+        makeRecordView(model,
                  controller='',
                  omittedFields=None,
                  additionalFields=None,
                  limitFields=None,
                  fieldAttrs=None,
-                 widgets=None, 
-                 validators=None, 
+                 widgets=None,
+                 validators=None,
                  formWidgetType=None,
                 )-->Toscawidgets.TableForm
-                 
+
         Generates an entry form for use in web applications.
-        
+
         Arguments:
         name              type                           default    Description
         model             class                                     sqlalchemy mapped class definition
@@ -263,7 +288,7 @@ class SprocketGenerator(DatabaseMixin):
         controller        String or None                 None       helps with doing links, possibly
         omittedFields     list of names                  []         fields that are not present, whatsoever
         additionalFields  list of Toscawidgets.fields    []         fields that you would like to add to the form that are not in the model
-        limitFields       list of names                  None       limit the fields to the one in the list.  A side effect is that the 
+        limitFields       list of names                  None       limit the fields to the one in the list.  A side effect is that the
                                                                     order of the fields is set by this argument.
                                                                     (a good way to order your fields if you include all of them)
         fieldAttrs        dict of field Attrs            {}         a set of extra parameters which can be passed into the widgets
@@ -280,7 +305,7 @@ class SprocketGenerator(DatabaseMixin):
             widgets={}
         if validators is None:
             validators={}
-        
+
         if not isinstance(omittedFields, types.ListType):
             raise TypeError('arg7(omittedFields) must be of type List, not %s'%type(omittedFields))
         if not isinstance(additionalFields, types.ListType):
@@ -289,11 +314,11 @@ class SprocketGenerator(DatabaseMixin):
             raise TypeError('arg9(limitFields) must be of type List, not %s'%type(limitFields))
         if not isinstance(fieldAttrs, types.DictType):
             raise TypeError('arg10(fieldAttrs) must be of type Dict, not %s'%type(limitFields))
-        
+
         self._validateModel(model)
-        
-        view = self.makeView(RecordViewConfig, 
-                        model, 
+
+        view = self.makeView(RecordViewConfig,
+                        model,
                         '',
                         '',
                         controller,
@@ -306,19 +331,19 @@ class SprocketGenerator(DatabaseMixin):
                          )
         return view.widget
 
-    def makeView(self, viewConfigType, 
-                 model, 
-                 action='', 
+    def makeView(self, viewConfigType,
+                 model,
+                 action='',
                  identifier=None,
                  controller='/',
-                 hiddenFields=None, 
+                 hiddenFields=None,
                  disabledFields=None,
                  requiredFields=None,
-                 omittedFields=None, 
+                 omittedFields=None,
                  additionalFields=None,
                  limitFields=None,
                  fieldAttrs=None,
-                 widgets=None, 
+                 widgets=None,
                  validators=None,
                  widgetType=None,
                  formValidator=None,
@@ -326,7 +351,7 @@ class SprocketGenerator(DatabaseMixin):
                  makeLinks=None
                  ):
         """this function was added so that you could cache the views more easily if you wanted to use the dbsprockets caching system"""
-        
+
         if hiddenFields is None:
             hiddenFields = []
         if disabledFields is None:
@@ -373,15 +398,15 @@ class SprocketGenerator(DatabaseMixin):
             tipe = type(viewConfig)
             method = new.instancemethod(makeLinks, viewConfig, tipe)
             viewConfig._makeLinks = method
-            
+
         view = ViewFactory().create(viewConfig)
         return view
-    
-    def makeTable(self, model, 
+
+    def makeTable(self, model,
                   controller=None,
                   identifier='',
-                  omittedFields=None, 
-                  additionalFields=None, 
+                  omittedFields=None,
+                  additionalFields=None,
                   limitFields=None,
                   widgetType=None,
                   makeLinks=None,
@@ -396,9 +421,9 @@ class SprocketGenerator(DatabaseMixin):
                  widetType=None,
                  makeLinks=None,
                  )-->Toscawidgets.DataGrid
-                 
+
         Generates a datagrid widget for population of database data
-        
+
         Arguments:
         name              type                           default    Description
         model             class                                     sqlalchemy mapped class definition
@@ -407,8 +432,8 @@ class SprocketGenerator(DatabaseMixin):
         omittedFields     list of names                  []         fields that are not present, whatsoever.
                                                                     if '_action' is in the list, then the edit/delete links will be removed.
         additionalFields  list of Toscawidgets.fields    []         fields that you would like to add to the form that are not in the model
-        limitFields       list of names                  None       limit the fields to the one in the list.  A side effect is that the 
-                                                                    order of the fields is set by this argument.  
+        limitFields       list of names                  None       limit the fields to the one in the list.  A side effect is that the
+                                                                    order of the fields is set by this argument.
                                                                     (a good way to order your fields if you include all of them)
         widgetType        ToscaWidget                    None       field for the parent widget to be used
         makeLinks         function                       None       Add a custom method for creating action links in your table.
@@ -417,16 +442,16 @@ class SprocketGenerator(DatabaseMixin):
             omittedFields=[]
         if additionalFields is None:
             additionalFields=[]
-        
+
         self._validateModel(model)
         if controller is None:
             controller = ''
             omittedFields.append('_actions')
 
-        view = self.makeView(TableViewConfig, 
-                        model, 
-                        controller=controller, 
-                        omittedFields=omittedFields, 
+        view = self.makeView(TableViewConfig,
+                        model,
+                        controller=controller,
+                        omittedFields=omittedFields,
                         additionalFields=additionalFields,
                         widgetType=widgetType,
                         makeLinks=makeLinks,
@@ -434,14 +459,14 @@ class SprocketGenerator(DatabaseMixin):
         return view.widget
 
 class DataFetcher(DatabaseMixin):
-        
+
     def getTableValue(self, model, offset=None, limit=None, foreignKeyFieldLabels={}):
         """
         getTableValue(model) --> SQLAlchemy result rows
         gets a set of results for a given model
-        
+
         xxx: limit and offset are currently not employed
-        
+
         Arguments:
         name              type                           Description
         model             class                          sqlalchemy mapped class definition
@@ -458,7 +483,7 @@ class DataFetcher(DatabaseMixin):
         """
         getTableValue(model) --> SQLAlchemy result rows
         gets a set of results for a given model
-        
+
         Arguments:
         name              type                           Description
         model             class                          sqlalchemy mapped class definition
@@ -468,7 +493,7 @@ class DataFetcher(DatabaseMixin):
         identifier = helper.getIdentifier(model)
         provider = helper.getProvider(model)
         return provider.getDefaultValues(identifier)
- 
+
 generator = SprocketGenerator()
 fetcher = DataFetcher()
 
